#pragma once

// Genode includes
#include <base/rpc.h>
#include <dataspace/capability.h>
#include <session/session.h>

namespace Timestamp {
	using Genode::uint64_t;
	using namespace Genode;

	enum {
		DEFAULT_BUFFER_SIZE = 8*1024,           // maximal size in bytes
		CERTIFICATE_SIZE = DEFAULT_BUFFER_SIZE  // maximal size in bytes
	};

	struct Session;

	class Buffer_too_small  : public Genode::Exception { };
	class Hsm_error         : public Genode::Exception { };
	class Invalid_input     : public Genode::Exception { };
}

struct Timestamp::Session : public Genode::Session
{
	/**
	 * \noapi
	 */
	static const char *service_name() { return "Timestamp"; }

	enum { CAP_QUOTA = 3 };

	/**
	 * Get the timestamp in seconds since
	 * \result                    the timestamp in seconds since epoch
	 *
	 * \throw Error_response_exception
	 * \throw Buffer_too_short_exception
	 */
	virtual uint64_t get() const = 0;

	/**
	 * Get the timestamp in seconds since epoch and in RFC 3161 form
	 *
	 * \param hash_buf            the hash for RFC 3161 timestamp signature
	 * \param hash_buf_size       the size of the hash
	 * \param rfc3161_buf         the out buffer for RFC 3161 timestamp
	 * \param rfc3161_buf_size    the size of the buffer for RFC 3161 timestamp
	 * \param rfc3161_length      the length of the RFC 3161 timestamp, 0 meaning invalid
	 * \result                    the timestamp in seconds since epoch
	 *
	 * \throw Error_response_exception
	 * \throw Buffer_too_short_exception
	 */
	virtual uint64_t get(uint8_t const *hash_buf,
	                     size_t         hash_buf_size,
	                     uint8_t       *rfc3161_buf,
	                     size_t         rfc3161_buf_size,
	                     size_t        &rfc3161_length) = 0;

	/**
	 * Get the certificate used for the RFC 3161 timestamp.
	 *
	 * \param rfc3161_cert_buf         the out buffer for RFC 3161 timestamp certificate
	 * \param rfc3161_cert_buf_size    the size of the buffer for RFC 3161 timestamp certificate
	 * \result                         the size of the RFC 3161 timestamp certificate, 0 meaning invalid
	 */
	virtual size_t get_rfc3161_cert(void *rfc3161_cert_buf, size_t rfc3161_cert_buf_size) = 0;

	/*******************
	 ** RPC interface **
	 *******************/

	GENODE_RPC_THROW(Rpc_get, uint64_t, get,
	                 GENODE_TYPE_LIST(Hsm_error, Buffer_too_small));
	GENODE_RPC_THROW(Rpc_get_rfc3161, uint64_t, _get_rfc3161,
	                 GENODE_TYPE_LIST(Hsm_error, Buffer_too_small), size_t, size_t&);
	GENODE_RPC_THROW(Rpc_get_rfc3161_cert, size_t, _get_rfc3161_cert,
	                 GENODE_TYPE_LIST(Hsm_error, Buffer_too_small), size_t);

	GENODE_RPC(Rpc_dataspace, Dataspace_capability, _dataspace);

	GENODE_RPC_INTERFACE(Rpc_get, Rpc_get_rfc3161, Rpc_get_rfc3161_cert, Rpc_dataspace);
};

